package iscrtavanjeGrafa;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.JFrame;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

public class OsnovniFrejm extends JFrame {

    //nasledjeni JPanel koji ce nam sluziti za crtanje grafa
    GrafOkvir okvirZaCrtanje;

    //koristimo "obicnu" JTable klasu, ali ce model biti nasledjen
    JTable grafTabela;

    public OsnovniFrejm() {

	//standardna podesavanja
	setPreferredSize(new Dimension(500, 500));
	setVisible(true);
	setDefaultCloseOperation(EXIT_ON_CLOSE);
	dodajKomponente();
	pack();
    }

    void dodajKomponente() {

	//podesavamo grid layout koji se ponasa kao tabela
	//u html-u, pritom definisemo broj redova i kolona
	setLayout(new GridLayout(2,1));

	okvirZaCrtanje = new GrafOkvir();

	//modeli za tabele u javi su generalno najkompleksniji
	//kada su u pitanju mogucnosti obavestavanja o dogadjajima
	//kao sto su npr. promena strukuture, umetanje i obavestavanje
	//o promenama na ciljanim redovima i kolonama, izmene podataka
	//i slicno..
	//mi koristimo nasledjeni model iz razloga sto nam je korisno
	//da podatke o grafovima odrzavamo na specifican nacin, a to
	//je spisak cvorova i matrica povezanosti
	GrafTableModel model = new GrafTableModel();

	//modelu dodeljujemo klasu koja ce ga osluskivati u slucaju neke
	//promene, a to je svakako prostor za iscrtavanje grafa, jer 
	//ce od sadrzaja modela tabele zavisiti i kakav ce graf biti nacrtan
	model.addTableModelListener(okvirZaCrtanje);

	grafTabela = new JTable(model);

	//dodajemo najpre deo za crtanje, jer se grid layout
	//popunjava redom sa leva na desno odozgo na dole
	getContentPane().add(okvirZaCrtanje);

	getContentPane().add(grafTabela);

	//dodeljujemo osluskivac za akcije misem, interesantan nam je klik
	//jer je tada potrebno dodati novi cvor u model tabele, samim tim
	//izmeniti joj i strukturu
	okvirZaCrtanje.addMouseListener(new MouseAdapter() {
	    @Override
	    public void mouseClicked(MouseEvent e) {
		//iz dogadjaja izvlacimo posiljaoca dogadjaja, a to je 
		//su informacije o poziciji misa u momentu klika i slicno
		// i na osnovu toga formiramo cvor i dodajemo ga modelu
		GrafTableModel model=(GrafTableModel)grafTabela.getModel();
		model.dodajCvor(new Cvor(e.getPoint().x, e.getPoint().y));
	    }
	});

    }

    /**
     * @param args
     */
    public static void main(String[] args) {
	// TODO Auto-generated method stub
	OsnovniFrejm of = new OsnovniFrejm();

    }

}

class GrafTableModel extends DefaultTableModel {

    //lista cvorova
    ArrayList<Cvor> cvorovi = new ArrayList<Cvor>();

    //matrica povezanosti
    int[][] mat;

    public GrafTableModel() {
	// TODO Auto-generated constructor stub
    }

    void dodajCvor(Cvor cvor) {

	cvorovi.add(cvor);
	//ako matrica nije prazna, treba je prosiriti u 
	//smislu dodavanja jednog reda i jedne kolone
	if (mat != null) {
	    int[][] temp;
	    temp = new int[mat.length + 1][mat[0].length + 1];
	    for (int i = 0; i < mat.length; i++)
		for (int j = 0; j < mat[i].length; j++)
		    temp[i][j] = mat[i][j];
	    mat = temp;
	}else{
	    mat=new int[1][1];
	}

	//na dodate redove i kolone podesavamo nule, jer to 
	//znaci da grana jos nije definisana
	for (int i = 0; i < mat.length; i++) {
	    mat[mat.length - 1][i] = 0;
	    mat[i][mat.length - 1] = 0;
	}

	//obavestavamo na ovaj nacin tabelu da je dodata kolona
	//i red i na taj nacin izazivamo novo iscrtavanje tabele
	addColumn("");
	fireTableRowsInserted(mat.length-1, mat.length);
	//ovim obavestavamo sve "nakacene" osluskivace, a to je 
	//u nasem slucaju jedan - sam panel za iscrtavanje grafa
	//da izvrsi promene koje ce biti u skladu sa novim modelom
	fireTableStructureChanged();
    }

    @Override
    public void setValueAt(Object value, int row, int column) {
        //ako ne definisemo drugacije, vrednosti ce biti pamcene
	//samo u generickim redovima i kolonama same tabele, mi smo
	//sa druge strane podatke vezali za nase zasebne strukture,
	//tj. za matricu povezanosti, samim tim moramo da u slucaju
	//promene od strane korisnika u samoj tabeli te promene
	//azuriramo i u matrici povezanosti
	mat[row][column]=Integer.valueOf(value.toString());
	mat[column][row]=Integer.valueOf(value.toString());
        super.setValueAt(value, row, column);
    }
    
    @Override
    public int getRowCount() {
	//vrlo je  bitno redefinisati ovaj metod
	//i vezati ga za nase specificno stanje podataka
	// a to je lista cvorova
	if(cvorovi!=null)
	    return cvorovi.size();
	return 0;
    }

    @Override
    public Object getValueAt(int row, int column) {
	//vrlo je bitno redefinisati ovaj metod
	//takodje model mora da zna koju vrednost da vrati
	//kada mu se neko obrati za vrednost na nekoj poziciji,
	//mi koristimo vrednost iz matrice povezanosti
	if(mat!=null)
	    return mat[row][column];
	return null;
    }
}